package com.bweb.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.config.PetroConfig;
import com.hai.entity.ApiCoupon;
import com.hai.model.ResponseData;
import com.hai.openApi.service.ApiBlxCouponService;
import com.hai.openApi.service.ApiCouponService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;

@Controller
@RequestMapping(value = "/apiCoupon")
@Api(value = "外部卡券查询")
public class ApiCouponController {

    private static Logger log = LoggerFactory.getLogger(HighDiscountPackageController.class);

    @Resource
    private ApiBlxCouponService apiBlxCouponService;

    @Resource
    private ApiCouponService apiCouponService;



    @RequestMapping(value = "/getBlxCouponList", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "获取比邻星停车券列表")
    public ResponseData getBlxCouponList( @RequestParam(name = "pageNum", required = true) Integer pageNum,
                                          @RequestParam(name = "pageSize", required = true) Integer pageSize) {
        try {

            PageHelper.startPage(pageNum, pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(apiBlxCouponService.getBlxCouponList(new HashMap<>())));

        } catch (Exception e) {
            log.error("HighDiscountPackageController --> getDiscountPackageList() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/synCouponRule", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "获取中石油卡券")
    public ResponseData synCouponRule(@RequestParam(name = "type", required = true) Integer type) {
        try {

            PetroConfig.init(type);

            JSONObject jsonObject = PetroConfig.synCouponRule();

            if (jsonObject.getString("code").equals("1")) {
                String data = PetroConfig.decrypt(jsonObject.getString("biz_content"));
                JSONArray jsonArray = JSONArray.parseArray(data);

                System.out.println(jsonArray);
                ApiCoupon apiCoupon;

                for (int i = 0; i < jsonArray.size(); i++) {

                    JSONObject object = jsonArray.getJSONObject(i);
                    apiCoupon = apiCouponService.findCoupon(object.getString("request_code"));

                    System.out.println(object);
                    System.out.println(object.getString("request_code"));
                    System.out.println(apiCoupon);



                    if (apiCoupon == null ) {
                        if (object.getInteger("status") == 1 && object.getInteger("total_num") > 0) {
                            apiCoupon = new ApiCoupon();
                            apiCoupon.setCode(object.getString("request_code"));
                            apiCoupon.setDesc(object.getString("ticket_desc"));
                            apiCoupon.setGoodsType(object.getString("goods_type"));
                            apiCoupon.setName(object.getString("ticket_name"));
                            apiCoupon.setEffectiveDays(object.getInteger("effective_days"));
                            apiCoupon.setImg(object.getString("img"));
                            apiCoupon.setCreateTime(new Date());
                            apiCoupon.setUpdateTime(new Date());
                            apiCoupon.setStatus(1);
                            apiCoupon.setType(type);
                            apiCoupon.setPrice(object.getInteger("face_amount"));
                            apiCouponService.insertCoupon(apiCoupon);
                        }

                    } else {
                        apiCoupon.setCode(object.getString("request_code"));
                        apiCoupon.setDesc(object.getString("ticket_desc"));
                        apiCoupon.setName(object.getString("ticket_name"));
                        apiCoupon.setGoodsType(object.getString("goods_type"));
                        apiCoupon.setEffectiveDays(object.getInteger("effective_days"));
                        apiCoupon.setImg(object.getString("img"));
                        apiCoupon.setUpdateTime(new Date());
                        apiCoupon.setPrice(object.getInteger("face_amount"));
                        apiCoupon.setType(type);
                        apiCoupon.setStatus(object.getInteger("status"));
                        apiCouponService.editCoupon(apiCoupon);
                    }
                }


                return ResponseMsgUtil.success(jsonArray);
            }
            return ResponseMsgUtil.success("请求失败");

        } catch (Exception e) {
            log.error("HighDiscountPackageController --> getDiscountPackageList() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

}
